#! /usr/bin/env bash

# TODO(achanda): Combine this constant and the one in nsx-agent-init.cc in one
# place.
nsx_group="nsx"
nsx_nestdb_group="nestdb"
nsx_netcpa_group="netcpa"

nsx_agent_user="nsx-agent"
nsx_agent_group="nsx-agent"

NSX_AGENT_TAG=NSX-NsxAgent
NSX_AGENT_MAX_QUICK_RESTARTS="10"
NSX_AGENT_MAX_RESTARTS="1000"
WATCHDOG_ARGS="-d -s ${NSX_AGENT_TAG} -q ${NSX_AGENT_MAX_QUICK_RESTARTS} -t ${NSX_AGENT_MAX_RESTARTS}"
WATCHDOG=/opt/vmware/nsx-agent/bin/watchdog.sh

# Create the nsx_agent_user user and group.  nsx-agent daemon would drop root
# user id and switch to this id after it boots up.
groupadd -f $nsx_group
groupadd -f $nsx_nestdb_group
groupadd -f $nsx_netcpa_group
groupadd -f $nsx_agent_group
if ! getent passwd $nsx_agent_user > /dev/null; then
  useradd --system -N \
    -g $nsx_agent_group \
    --shell /usr/sbin/nologin -M \
    --comment "NSX Agent" $nsx_agent_user
fi
usermod -a -G $nsx_group,$nsx_nestdb_group,$nsx_netcpa_group $nsx_agent_user

if [ -f /etc/default/nsx-agent ] ; then
    # load this once, at the beginning, to make sure core and fd limits get
    # pulled in.
    . /etc/default/nsx-agent
fi

if test -n "$ONIX_CORE_LIMIT"; then
    echo "Setting core limit to $ONIX_CORE_LIMIT"
    ulimit -c "$ONIX_CORE_LIMIT" || true
fi

if test -n "$ONIX_FD_LIMIT"; then
    echo "Setting file descriptor limit to $ONIX_FD_LIMIT"
    ulimit -n "$ONIX_FD_LIMIT" || true
fi

out=`grep $nsx_agent_user /etc/sudoers`
if [ -z "$out" ]; then
    echo "nsx-agent ALL=(ALL)NOPASSWD:/usr/bin/ovs-appctl" >> /etc/sudoers
    echo "nsx-agent ALL=(ALL)NOPASSWD:/usr/bin/ovs-vsctl" >> /etc/sudoers
    echo "Defaults:nsx-agent !requiretty" >> /etc/sudoers
fi

CLOUDNET_PATH=/opt/vmware/nsx-agent/bin
CONTROLLERD="$CLOUDNET_PATH/nsx-agent-daemon"
HEARTBEAT_INTERVAL=100
STALL_THRESH=3
NAME=nsx-agent
NSX_AGENT_RUN_DIR=/var/run/vmware/nsx-agent
PIDFILE=$NSX_AGENT_RUN_DIR/$NAME.pid
#This file captures the exit status of nsx-agent-daemon
STATUSFILE=$NSX_AGENT_RUN_DIR/$NAME.status
CHILD_PIDFILE=/var/run/vmware/nsx-agent/$NAME-domains.pid
PARENT_LOGDIR=/var/log/vmware
LOGDIR=$PARENT_LOGDIR/nsx-agent
CLOUDNET_DATA_DIR=/var/vmware/nsx-agent/data

. /usr/share/openvswitch/scripts/ovs-lib || exit 1

# Punch holes in the firewall for gre, stt, vxlan
# and geneve.
ovs_ctl --protocol=gre enable-protocol
ovs_ctl --protocol=tcp --dport=7471 enable-protocol
ovs_ctl --protocol=udp --dport=4789 enable-protocol
ovs_ctl --protocol=udp --dport=6081 enable-protocol

# Export VERBOSE_LEVEL to indicate loglevel was set by it so that
# we don't override API settings with GLOG_v as is done in the
# development environment.
export VERBOSE_LEVEL=$VERBOSE_LEVEL

mkdir -p $LOGDIR
chmod 775 $PARENT_LOGDIR
chmod -R 700 $LOGDIR
chown -R $nsx_agent_user:$nsx_agent_group $LOGDIR
export GLOG_v=$VERBOSE_LEVEL
export GLOG_log_dir=$LOGDIR
export GLOG_logtostderr=0
export GLOG_alsologtostderr=0
export GLOG_stderrthreshold=2
export GLOG_max_log_size=100
export GLOG_stop_logging_if_full_disk=1

# HyperBus
HYPERBUSCMD=/opt/vmware/nsx-agent/bin/hyperbus-cmd
HYPERBUS_SOCK_DIR=/var/run/vmware/hyperbus
CNI_SOCK_DIR=/var/run/nsx-ujo
HYPERBUS_CFG_FILE=/etc/vmware/nsx/hyperbus.xml

mkdir -p "${HYPERBUS_SOCK_DIR}"
chmod 0770 "${HYPERBUS_SOCK_DIR}"
chown -R $nsx_agent_user:$nsx_nestdb_group "${HYPERBUS_SOCK_DIR}"
mkdir -p "${CNI_SOCK_DIR}"
chmod 0770 "${CNI_SOCK_DIR}"
chown -R $nsx_agent_user:$nsx_nestdb_group "${CNI_SOCK_DIR}"
chmod 640 "${HYPERBUS_CFG_FILE}"
chown $nsx_agent_user:$nsx_nestdb_group "${HYPERBUS_CFG_FILE}"

# setuid flag for hyperbus-cmd
chmod o-x $HYPERBUSCMD
chmod g+x $HYPERBUSCMD

ln -s /etc/os-release /etc/vmware/nsx/os-release
if [ -f /etc/centos-release ]; then
    ln -s /etc/centos-release /etc/vmware/nsx/centos-release
fi

# Turn on profiling options
export NVP_NLOG_PROFILE_MONITOR=1
export ONIX_MEMSTATS_ENABLE="1"
export ONIX_MEMSTATS_INTERVAL_SEC="30"
export ONIX_HEAP_SAMPLE_INTERVAL_SEC="1800"
export ONIX_HEAP_SAMPLE_PREFIX="/var/log/vmware/nsx-agent/onix_heap_sample"
export TCMALLOC_SAMPLE_PARAMETER="524288"

# XXX: use of c++ protobuf bindings in python is not done (bug 6493).
#export PROTOCOL_BUFFERS_PYTHON_IMPLEMENTATION=cpp

export monitor_log_export_onix_core_scheduler_profiler=300
export monitor_log_export_onix_core_rpc_stats=300
export nvp_phys_chassis_connected_unfiltered_hyst_hysteresis_delete_delay_ms=3600000
export nvp_phys_switch_connected_unfiltered_hyst_hysteresis_delete_delay_ms=3600000
export nvp_phys_switch_flow_sync_yield_count=1000
export nvp_phys_switch_flow_pending_delay_master_unknown_ms=2147483647

# ensure the path existence since /var/run is not persistent after reboot
mkdir -p $NSX_AGENT_RUN_DIR
chmod 775 $NSX_AGENT_RUN_DIR
chown -R $nsx_agent_user:$nsx_agent_group $NSX_AGENT_RUN_DIR

# Add capabilities to do xfrm netlink for conntrack, to do bind for syslog
setcap cap_net_bind_service,cap_sys_admin,cap_net_admin=eip $CONTROLLERD

# Create a symbolic link of the nsxa-ctl-server socket to
# /var/run/openvswitch.
# TODO: Remove this once all daemons have been converted
# to use the real path of the socket.
mkdir -p /var/run/openvswitch
ln -s $NSX_AGENT_RUN_DIR/nsxa-ctl /var/run/openvswitch/nsxa-ctl
ln -s $NSX_AGENT_RUN_DIR/vsip.sock /var/run/openvswitch/vsip.sock

mkdir -p $CLOUDNET_DATA_DIR
chown -R $nsx_agent_user:$nsx_agent_group $CLOUDNET_DATA_DIR

# During nsx-agent upgrade, nsx-agent is restarted before the previous
# files are deleted. This can cause nsx-agent to core dump since
# the same app is declared in multiple meta.json file (in old meta.json
# and new meta.json).
# As a workaround, explicitly delete these files during start up.
rm -rf /opt/vmware/nsx-agent/bin/cpp/nsx-agent/netcpa/dfw/meta.json
rm -rf /opt/vmware/nsx-agent/bin/cpp/nsx-agent/nestdb/l2/meta.json
rm -rf /opt/vmware/nsx-agent/bin/cpp/nsx-agent/nestdb/l3/meta.json
rm -rf /opt/vmware/nsx-agent/bin/cpp/nsx-agent/netcpa/traceflow/meta.json

cd $CLOUDNET_PATH

# Obtain the pid
pid=`cat $PIDFILE`

# Finds all process that matches ${CONTROLLERD} and kills it..
ps -ef | grep ${CONTROLLERD} | grep -v grep | awk '{print $2}' | xargs -r kill -s 9

# start only C++ domain
NESTDB_ARGS="""`output_comp nestdb-channel enable_nestdb=true,enable_lcp_nestdb=true`"""

# Use runuser on Ubuntu host only
if [ `grep 'Ubuntu' /etc/os-release | wc -l` -ne 0 ]; then
    runuser -m $nsx_agent_user -s /bin/bash \
        -c "${WATCHDOG} ${WATCHDOG_ARGS} '$NSX_AGENT_EXEC_PREFIX $CONTROLLERD -p $PIDFILE $CLOUDNET_ARGS $NESTDB_ARGS' >/dev/null 2>&1"
else
    su -m $nsx_agent_user -s /bin/bash -c "$NSX_AGENT_EXEC_PREFIX $CONTROLLERD \
        -p $PIDFILE $CLOUDNET_ARGS $NESTDB_ARGS &"
fi

# wait up to 15 seconds for nsx-agent to start
for i in `seq 0 15`; do
    sleep 1
    curr_pid=`cat $PIDFILE`
    if [ "$pid" != "$curr_pid" ] ; then
      break
    fi
done
