# **********************************************************************
# Copyright 2017-2018 VMware, Inc. All rights reserved.
# -- VMware Confidential
# **********************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# **********************************************************************
import json
import nsx_logging
from cli.utils.command_utils import invoke_command
from cli.utils.node_utils import get_node_type
from cli.utils.constants import NODE_TYPE_HOST_CENTOSKVM
from cli.utils.constants import NODE_TYPE_HOST_SLESKVM
from cli.utils.constants import NODE_TYPE_HOST_ESX
from cli.utils.constants import NODE_TYPE_HOST_RHELKVM
from cli.utils.constants import NODE_TYPE_HOST_UBUNTUKVM
from cli.utils.constants import NODE_TYPE_HOST_WIN
from cli.utils.constants import NODE_TYPE_HOST_PUBLIC_CLOUD
from cli.utils.constants import NODE_TYPE_BAREMETAL_UBUNTU_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_RHEL_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_CENTOS_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_SLES_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_OEL_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_WINDOWS_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_RHEL_CONTAINER
from cli.utils.constants import NODE_TYPE_BAREMETAL_CENTOS_CONTAINER
from cli.utils.location_utils import FilePaths
from cli.utils.error_code import ErrorCode as ec
from cli.commands.cli_command_provider import CliCommandProvider

lg = nsx_logging.getLogger("cli.commands.host_shared.firewallipfix")


#
# DFW IPFIX related commands
#
_FIREWALL_IPFIX_CLI = {
    NODE_TYPE_HOST_SLESKVM: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_HOST_CENTOSKVM: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_HOST_ESX: {
        'bin': "/opt/vmware/nsx-cli/bin/nsx-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_HOST_RHELKVM: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_HOST_UBUNTUKVM: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_HOST_PUBLIC_CLOUD: {
        'bin': FilePaths.OVS_APPCTL_PATH,
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_HOST_WIN: {
        'bin': FilePaths.OVS_APPCTL_PATH,
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_BAREMETAL_UBUNTU_SERVER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_BAREMETAL_RHEL_SERVER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_BAREMETAL_CENTOS_SERVER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_BAREMETAL_SLES_SERVER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_BAREMETAL_OEL_SERVER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_BAREMETAL_RHEL_CONTAINER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_BAREMETAL_CENTOS_CONTAINER: {
        'bin': "/usr/bin/ovs-appctl",
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    NODE_TYPE_BAREMETAL_WINDOWS_SERVER: {
        'bin': FilePaths.OVS_APPCTL_PATH,
        'socket': FilePaths.EXPORTER_CLI_SOCKET
        },
    }

def has_firewall_ipfix_cli(node_type):
    return (node_type == NODE_TYPE_HOST_SLESKVM or
            node_type == NODE_TYPE_HOST_CENTOSKVM or
            node_type == NODE_TYPE_HOST_ESX or
            node_type == NODE_TYPE_HOST_RHELKVM or
            node_type == NODE_TYPE_HOST_UBUNTUKVM or
            node_type == NODE_TYPE_HOST_PUBLIC_CLOUD or
            node_type == NODE_TYPE_BAREMETAL_UBUNTU_SERVER or
            node_type == NODE_TYPE_BAREMETAL_RHEL_SERVER or
            node_type == NODE_TYPE_BAREMETAL_CENTOS_SERVER or
            node_type == NODE_TYPE_BAREMETAL_OEL_SERVER or
            node_type == NODE_TYPE_BAREMETAL_SLES_SERVER or
            node_type == NODE_TYPE_BAREMETAL_WINDOWS_SERVER or
            node_type == NODE_TYPE_BAREMETAL_RHEL_CONTAINER or
            node_type == NODE_TYPE_BAREMETAL_CENTOS_CONTAINER or
            node_type == NODE_TYPE_HOST_WIN)

def invoke_firewall_ipfix_command(cmd_list):
    node_type = get_node_type()
    if has_firewall_ipfix_cli(node_type):
        cmd = [_FIREWALL_IPFIX_CLI[node_type]['bin'], "-t", _FIREWALL_IPFIX_CLI[node_type]['socket']]
        cmd.extend(cmd_list)
        rc, out, err = invoke_command(cmd)
        return (rc, out, err)
    else:
        lg.error("node type %s does not have firewall ipfix CLI", node_type,
                 error_code=ec.INVALID_ARGUMENT)
        return (-1, None, "invalid command")

def get_firewall_ipfix_cli_output(command):
    cmd_list = command
    rc, out, err = invoke_firewall_ipfix_command(cmd_list)
    if rc != 0:
        lg.error("Failed to invoke firewall ipfix CLI command, rc: %d, err: %s", rc, err,
                 error_code=ec.OPERATION_FAILED)
        out = None

    return out

class FirewallIpfixShowBase(CliCommandProvider):
    CMD_STR = None
    def __call__(self, cmd_list):
        lg.debug("%s: cmd_list cnt=%d" % (self.__class__.__name__, len(cmd_list)))
        if self.CMD_STR:
            cmds = [self.CMD_STR]
            if (len(cmd_list) > 3):
                cmds.extend([cmd_list[3]])
            res = get_firewall_ipfix_cli_output(cmds)
            cmd_result = ""
            if res is None:
                cmd_result = "MSG_FIREWALLIPFIX_COMM_ERROR"
            elif "not available" in res:
                cmd_result = "MSG_FIREWALLIPFIX_NOT_AVAILABLE"
            else:
                cmd_result = "OK\n\n%s" % res
        else:
            cmd_result = "MSG_FIREWALLIPFIX_COMM_ERROR"

        return cmd_result

class FirewallShowRuleStats(FirewallIpfixShowBase):
    CMD_STR = "get/rulestats"

class FirewallShowRuleStatsTotal(FirewallIpfixShowBase):
    CMD_STR = "get/rulestats"

class FirewallShowIpfixConfig(FirewallIpfixShowBase):
    CMD_STR = "get/profiles"

class FirewallShowIpfixStats(FirewallIpfixShowBase):
    CMD_STR = "get/ipfixstats"

class FirewallShowIpfixContainers(FirewallIpfixShowBase):
    CMD_STR = "get/container"

class FirewallShowIpfixFilters(CliCommandProvider):
    CMD_STR = "get/filters"
    def __call__(self, cmd_list):
       res = get_firewall_ipfix_cli_output(["get/filters"])
       cmd_result = ""
       if res is None:
           cmd_result = "MSG_FIREWALLIPFIX_COMM_ERROR"
       elif "not available" in res:
           cmd_result = "MSG_FIREWALLIPFIX_NOT_AVAILABLE"
       else:
           # Need to reformat results to keep only vifs
           resjson = json.loads(res)
           vifs = resjson.get("vifs")
           newres = { "vifs":vifs }
           cmd_result = "OK\n\n%s" % json.dumps(newres)

       return cmd_result

class FirewallShowIpfixVifmap(FirewallIpfixShowBase):
    CMD_STR = "get/vifmap"

