# ***************************************************************************
# Copyright 2019 VMware, Inc.  All rights reserved. VMware Confidential.
# ***************************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# ***************************************************************************
import nsx_logging
import json

from cli.commands.cli_command_provider import CliCommandProvider
from cli.commands.lcp_common.cfgagent.client import get_cfgagent_cli_output
from cli.utils.error_code import ErrorCode as ec

lg = nsx_logging.getLogger("cli.commands.lcp_common.cfgagent.ipfix")


def load_response_output(response):
    try:
        if response is None:
            return None
        output = json.loads(response)
    except (TypeError, ValueError) as err:
        lg.error("load response error: %s", err,
                 error_code=ec.INTERNAL_ERROR)
        return None
    return output


class GetCfgagentIpfixTable(CliCommandProvider):
    def __call__(self, cmd_list):
        response = get_cfgagent_cli_output("cfgagent/ipfix-switch/profile/all")
        output = load_response_output(response)
        if output is None:
            return "CFGAGENT_IPFIX_TABLE_FAILED"

        return "OK\n\n%s" % json.dumps(output)


class GetCfgagentIpfixProfileByLSP(CliCommandProvider):
    def __call__(self, cmd_list):
        uuid = cmd_list[6]
        response = get_cfgagent_cli_output("cfgagent/ipfix-switch/profile/lport", [uuid])
        output = load_response_output(response)
        if output is None:
            return "CFGAGENT_IPFIX_TABLE_FAILED"

        return "OK\n\n%s" % json.dumps(output)


