# **********************************************************************
# Copyright 2014 VMware, Inc.  All rights reserved. VMware Confidential.
# **********************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# **********************************************************************


__all__ = ["CliHelpSummaryDescriptor"]


import nsx_logging
import os
import yaml

from lib.json_schema import JsonSchema
from cli.descriptors.descriptor import Descriptor
from cli.descriptors.descriptor import SCHEMA_REGISTRY


lg = nsx_logging.getLogger("cli.descriptors.cli_help_summary_descriptor")


CLI_BASE_DIR = os.path.dirname(os.path.dirname(__file__))
CLI_HELP_SCHEMA_FILE = os.path.join(os.path.dirname(__file__),
                                       'cli_help_summary_descriptor.yml')
CLI_HELP_TYPES = dict((r.id, r)
                         for r in yaml.load(open(CLI_HELP_SCHEMA_FILE), Loader=yaml.Loader))
for key in list(CLI_HELP_TYPES.keys()):
    if key not in SCHEMA_REGISTRY:
        JsonSchema(CLI_HELP_TYPES[key].raw_schema,
                   schema_registry=SCHEMA_REGISTRY, type_path=[key],
                   strict=True)


class CliHelpSummaryDescriptor(Descriptor):
    """Descriptor specification of a CLI help summary.

    Help summaries are shown whenever a user enters ?, and are in the form of:
      id1    _help_summary for id1
      id2    _help_summary for id2
      ...
    """
    yaml_tag = '!CliHelpSummary'

    def __init__(self, id, schema_spec):
        super(CliHelpSummaryDescriptor, self).__init__(id, schema_spec)
        self._raw_schema = schema_spec
        self._help_summary = schema_spec["help_summary"]
        self._nsx_feature = schema_spec.get("nsx_feature")

    @property
    def help_summary(self):
        return self._help_summary
