﻿# **********************************************************************
# Copyright 2015-2019 VMware, Inc.  All rights reserved. VMware Confidential.
# **********************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# **********************************************************************
import os
import sys
import subprocess

class FilePaths:
    if sys.platform == "win32":
        # TODO: Windows NSX-CLI needs to copy cli_config.conf to /NSX/cli
        # TODO: Due to node_utils.py importing this module, before logger gets
        # setup within nsxcli.py, any attempt to log throws "No handlers could
        # be found" error.  Temporarily disabling logging in this module to
        # avoid such error. The right fix is to setup logging before
        # calling/importing any module, which directly or as part of import
        # process may try to log.
        import os.path
        import nsx_logging
        from compat23.compat23_utils import compat23_winreg as winreg
        # lg = nsx_logging.getLogger("cli.utils.location_utils")
        NSX_REG_PATH = r"SOFTWARE\VMware, Inc.\NSX"
        try:
            hKey = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, NSX_REG_PATH)
            NSX_ROOT, regtype = winreg.QueryValueEx(hKey, "InstallDir")
            if regtype != winreg.REG_SZ:
                raise TypeError("NSX installation dir registry is not a string")
        except:
            NSX_ROOT = os.environ.get("NSX_ROOT") or "C:/NSX"
            # lg.exception("Unable to read registry %s, using default location %s",
            #              NSX_REG_PATH, NSX_ROOT)
        try:
            hKey = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, NSX_REG_PATH)
            NSX_DATA_DIR, regtype = winreg.QueryValueEx(hKey, "DataDir")
            if regtype != winreg.REG_SZ:
                raise TypeError("NSX data dir registry is not a string")
        except:
            NSX_DATA_DIR = os.path.join(os.getenv('PROGRAMDATA'), "VMware\\NSX\\Data")
            # lg.exception("Unable to read registry %s, using default location %s",
            #              NSX_REG_PATH, NSX_DATA_DIR)
        try:
            hKey = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, NSX_REG_PATH)
            NSX_LOG_DIR, regtype = winreg.QueryValueEx(hKey, "LogDir")
            if regtype != winreg.REG_SZ:
                raise TypeError("NSX log dir registry is not a string")
        except:
            NSX_LOG_DIR = os.path.join(os.getenv('PROGRAMDATA'), "VMware\\NSX\\Logs")
        try:
            hKey = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, NSX_REG_PATH)
            NSX_FILE_STORE, regtype = winreg.QueryValueEx(hKey, "FileStore")
            if regtype != winreg.REG_SZ:
                raise TypeError("NSX FileStore registry is not a string")
        except:
            NSX_FILE_STORE = os.path.join(os.getenv('PROGRAMDATA'), "VMware\\NSX\\FileStore")
        try:
            hKey = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, NSX_REG_PATH)
            OVS_ROOT, regtype = winreg.QueryValueEx(hKey, "OvsDir")
            if regtype != winreg.REG_SZ:
                raise TypeError("OVS installation dir registry is not a "
                                "string")
        except:
            OVS_ROOT = "C:/openvswitch"
            # lg.exception("Unable to read registry %s, using default location %s",
            #              NSX_REG_PATH, OVS_ROOT)
        CLI_CONFIG_FILE = os.path.join(NSX_ROOT, "nsx-cli", "cli", "etc", "cli_config.conf")
        HV_CERT_FILE = os.path.join(NSX_DATA_DIR, "host-cert.pem")
        HV_PRIVKEY_FILE = os.path.join(NSX_DATA_DIR, "host-privkey.pem")
        MPA_CONFIG_FILE = os.path.join(NSX_ROOT, "mpaconfig.json")
        NSX_COMMON_DIR = NSX_DATA_DIR
        NSX_MPA_DIR = NSX_ROOT
        NSXA_PATH = os.path.join(NSX_ROOT, "nsxa.exe")
        OVS_RUN_DIR = os.path.join(OVS_ROOT, "var", "run")
        OVS_BIN_DIR = os.path.join(OVS_ROOT, "usr", "bin")
        OVS_APPCTL_PATH = os.path.join(OVS_BIN_DIR, "ovs-appctl.exe")
        NSXA_CTL_SOCKET = os.path.join(OVS_RUN_DIR, "openvswitch", "nsxa-ctl")
        EXPORTER_CLI_SOCKET = os.path.join(OVS_RUN_DIR, "openvswitch", "common-exporter-cli")
        NSX_ISSUE_FILE = os.path.join(NSX_ROOT, "nsx_issue")
        NSX_PUBLIC_CLOUD_CONFIG_FILE = os.path.join(NSX_DATA_DIR,
                                                    "public-cloud-config.txt")
        NSX_PUBLIC_CLOUD_BOOTSTRAPPER = os.path.join(NSX_ROOT,
                                                     "config-public-cloud.ps1")
        NSX_UPGRADE_SCRIPT = os.path.join(NSX_DATA_DIR, "nsx_install.ps1")
        SUPPORT_BUNDLE_SCRIPT = os.path.join(NSX_ROOT, "support_save.ps1")
        OPENSSL_PATH = os.path.join(NSX_ROOT, "nsx-proxy", "bin", "openssl.exe")
        APPLIANCE_INFO_FILE = os.path.join(NSX_ROOT, "appliance-info.xml")
        MPA_DECOM_STATE_FILE = os.path.join(NSX_ROOT, ".decom_state")
        NSX_NESTDB_CLI = os.path.join(NSX_ROOT, "nsx-nestdb", "nestdb-cli.bat")
        HOST_CFG_FILE = os.path.join(NSX_ROOT, "host-cfg.xml")
    else:
        CLI_CONFIG_FILE = "/config/vmware/nsx-cli/cli_config.conf"
        HOST_CLI_CONFIG_FILE = "/etc/vmware/nsx-cli/cli_config.conf"
        HV_CERT_FILE = "/etc/vmware/nsx/host-cert.pem"
        HV_PRIVKEY_FILE = "/etc/vmware/nsx/host-privkey.pem"
        MPA_CONFIG_FILE = "/etc/vmware/nsx-mpa/mpaconfig.json"
        MPA_INIT_SCRIPT = "/etc/init.d/nsx-mpa"
        NSX_COMMON_DIR = "/etc/vmware/nsx"
        NSX_OPSAGENT_INIT_SCRIPT = "/etc/init.d/nsx-opsagent"
        NSX_MPA_DIR = "/etc/vmware/nsx-mpa"
        NSXA_INIT_SCRIPT = "/etc/init.d/nsxa"
        OVS_RUN_DIR = "/var/run/"
        OVS_BIN_DIR = "/usr/bin/"
        OVS_APPCTL_PATH = os.path.join(OVS_BIN_DIR, "ovs-appctl")
        NSXA_CTL_SOCKET = os.path.join(OVS_RUN_DIR, "vmware", "nsx-agent", "nsxa-ctl")
        #Path varies based on Node type for linux appliances, set to None for now
        EXPORTER_CLI_SOCKET = os.path.join(OVS_RUN_DIR, "vmware", "exporter", "common-exporter-cli")
        OPENSSL_PATH = "/usr/bin/openssl"
        NSX_ISSUE_FILE = "/etc/nsx_issue"
        NSX_PUBLIC_CLOUD_CONFIG_FILE = os.path.join(NSX_COMMON_DIR, "public-cloud-config")
        NSX_UPGRADE_SCRIPT = "/opt/vmware/nsx-agent/bin/install_nsx_vm_agent_common.sh"
        NSX_FILE_STORE = "/var/vmware/nsx/file-store"
        SUPPORT_BUNDLE_SCRIPT = "/opt/vmware/nsx-agent/bin/support_save.py"
        RESET_NSX_SCRIPT = "/opt/vmware/nsx-cli/bin/scripts/delete_nsx_instance_from_host.sh"
        TEMP_RESET_NSX_SCRIPT = "/tmp/delete_nsx_instance_from_host.sh"
        APPLIANCE_INFO_FILE = "/etc/vmware/nsx/appliance-info.xml"
        NSX_PROXY_INIT_SCRIPT = "/etc/init.d/nsx-proxy"
        MPA_DECOM_STATE_FILE = "/etc/vmware/nsx-mpa/.decom_state"
        HOST_CFG_FILE = "/etc/vmware/nsx/host-cfg.xml"
        NSX_LOG_DIR = "/var/log"
