# Copyright (C) 2020 VMware, Inc. All rights reserved.
# -- VMware Confidential

from vmware.nsx.sha import sha_pb2

import sha.config
import time
from sha.contrib.action.abc_base import Action

from sha.constants import PID, USER_NAME, NICE, NAME, VMS, RSS, SHARED, CPU_PERCENT,\
    MEMORY_PERCENT, TOP_PROCESS_LIST, PROCESS, MEM_LIST, CPU_LIST, PERCENT, MPA_HUB
from sha.core.channel.provider.mpa_provider import ReportMsg

class Tpreport(Action):
    _logger = sha.config.get_logger(__name__)

    @classmethod
    def populate_pb_msg(cls, **kwargs):
        def set_process_info(msg, data):
            msg.process_id = str(data[PID])
            msg.user = data[USER_NAME]
            msg.nice_value = str(data[NICE])
            msg.command = data[NAME]
            msg.virtual_memory_size = data[VMS]
            msg.resident_memory_size = data[RSS]
            msg.shared_memory_size = data[SHARED]
            msg.cpu_usage = data[CPU_PERCENT]
            msg.memory_usage = data[MEMORY_PERCENT]

        report_msg = sha_pb2.ShaReportMsg()
        report_msg.oid.left = sha.config.HOST_ID_LOW
        report_msg.oid.right = sha.config.HOST_ID_HIGH
        top_process_list = kwargs[TOP_PROCESS_LIST] or []
        status_msg = report_msg.status.add()
        status_msg.type = sha_pb2.ShaStatusMsg.ShaStatusType.Value(PROCESS)
        if top_process_list:
            mem_list = top_process_list.get(MEM_LIST, [])
            cpu_list = top_process_list.get(CPU_LIST, [])
            for mem in mem_list:
                mem_detail_msg = status_msg.process_status.process_status_by_top_mem.add()
                set_process_info(mem_detail_msg, mem)

            for cpu in cpu_list:
                cpu_detail_msg = status_msg.process_status.process_status_by_top_cpu.add()
                set_process_info(cpu_detail_msg, cpu)

        return report_msg.SerializeToString()


    @classmethod
    def do(cls, hubs, *args):
        def kernel(**kwargs):
            cls._logger.debug('Tpreport - signaling, process info:{}, profile:{}'
                              .format(kwargs['top_process_list'], kwargs['profile']))

            pb_msg = cls.populate_pb_msg(**kwargs)
            hubs[MPA_HUB].send(ReportMsg(b'', pb_msg, b'com.vmware.nsx.sha.ShaReportMsg'))

        return kernel
