# Copyright (C) 2020 VMware, Inc. All rights reserved.
# -- VMware Confidential

import platform
import socket
import traceback
import sha.config
from sha.contrib.monitor.traceCollector.internal.spanProcessor import SpanProcessService
from sha.core.profile._global_config_resolver import GlobalConfigResolver

class Connection:
    """
    Connection to receive clients' connection.
    """
    SOCKET_ADDR = "127.0.0.1"
    SOCKET_PORT = 6832
    SOCKET_BUFFER_SIZE = 65535

    _logger = sha.config.get_logger(__name__)
    _sock = None
    _stop = True
    _queue = None

    @classmethod
    def start(cls, queue):
        cls._queue = queue
        cls._sock = socket.socket(socket.AF_INET,  socket.SOCK_DGRAM)
        cls._sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        cls._sock.bind((cls.SOCKET_ADDR, cls.SOCKET_PORT))
        cls._logger.info("TM:Connection sock {}".format(cls._sock))
        cls._stop = False

    @classmethod
    def is_connected(cls):
        return cls._sock is not None

    @classmethod
    def wait_for_data(cls):
        if not cls._sock:
            return None
        while not cls._stop:
            message = cls._sock.recvfrom(cls.SOCKET_BUFFER_SIZE)
            cls._logger.info("TM:new connection {}.{} arrives".format(message[1], len(message)))
            cls._logger.info("TM:new message {} arrives".format(message[0]))
            config = GlobalConfigResolver.get()
            cls._logger.info("TM:workflow tracing config {}".format(config))
            if config['trace_type'] == 1:
                cls._queue.put({"nsx.trace.spans": message[0]})
            elif config['trace_type'] == 3:
                processor = SpanProcessService(cls._queue)
                processor.process(message[0])

    @classmethod
    def close(cls):
        cls._stop = True
        if cls._sock:
            cls._sock.close()
            cls._sock = None
