# Copyright (C) 2020-2021 VMware, Inc. All rights reserved.
# -- VMware Confidential

def report_edge_fw_stats(edge_fw_stats, profile):
    if edge_fw_stats is None or edge_fw_stats['value'] is None:
        logger.info("NO EDGE_FW_STATS stats for exporting.")
        return

    logger.debug("EDGE_FW_STATS stats is {}".format(edge_fw_stats['value']))
    # Use specified source to export metrics to wavefront
    wavefront_source = edge_fw_stats['value'][0]
    timestamp = edge_fw_stats['timestamp']

    regex_counter = r'(packet|byte)-.*-(in|out)-(pass|drop)'
    reason_list = [
        "reason-3whs",
        "reason-alg",
        "reason-bad-offset",
        "reason-bad-timestamp",
        "reason-congestion",
        "reason-connection-limit",
        "reason-drop-by-loadbalancer",
        "reason-failed-to-copy-pkt",
        "reason-fragment",
        "reason-half-open-tcp-max",
        "reason-icmp-max",
        "reason-inactive",
        "reason-ip-option",
        "reason-memory",
        "reason-nat-conn-limit",
        "reason-nat64-no-frgm-support",
        "reason-normalize",
        "reason-other-max",
        "reason-proto-cksum",
        "reason-queued-frag",
        "reason-redirect-iface-null",
        "reason-rst-sent",
        "reason-short",
        "reason-spoofguard",
        "reason-src-limit",
        "reason-state-insert",
        "reason-state-limit",
        "reason-state-mismatch",
        "reason-state-reuse",
        "reason-synproxy",
        "reason-tcp-flags",
        "reason-tcp-seqnum",
        "reason-translation",
        "reason-tun-fail",
        "reason-udp-max",
        "reason-update-state",
    ]

    for lr, fw_stats in edge_fw_stats['value'][1].items():

        stats_data = {}
        fw_stats_name = "edge." + lr + ".firewall"

        # firewall stats
        if fw_stats is not None:
            if 'total-stats' in fw_stats:
                fw_total_stats = fw_stats['total-stats']
                if fw_total_stats is None:
                    continue
                stats_data['cps'] = fw_total_stats.get('cps', 0)

                #stats raw data
                stats_raw_data = {}
                for key, value in fw_total_stats.items():
                    if key in reason_list:
                        stats_raw_data[key] = value
                    elif re.match(regex_counter, key):
                        stats_raw_data[key.replace('byte', 'bytes').replace('packet', 'packets')] = value
                stats_data['stats'] = stats_raw_data

        if stats_data:
            precondition = data_store['metrics']['nsx.sha.precondition']
            if precondition and precondition['value']['wavefront']['reached']:
                Exportdata(data = {fw_stats_name: stats_data},
                           exports = ['wavefront'],
                           source = wavefront_source,
                           timestamp = timestamp)
            if precondition and precondition['value']['tsdb']['reached']:
                lr_type, lr_name = lr.split('.')
                entity_type = options_pb2.MetricsEntityType.Value('TIER1') if lr_type == 'T1' else \
                              options_pb2.MetricsEntityType.Value('TIER0')
                msg = metrics_agent_rpc_pb2.MetricsAgentMessage(
                              timestamp = int(timestamp),
                              entity = entity_type,
                              entity_id = str(fw_stats.get('mp_router_id', 0)))
                try:
                    tmp_msg = nsx_edge_fw_metrics_stats_pb2.NsxEdgeFwStats(obj_id = str(fw_stats.get('interface', 0)),
                              cps = stats_data['cps'],
                              drop_reason_3whs = stats_raw_data['reason-3whs'],
                              drop_reason_alg = stats_raw_data['reason-alg'],
                              drop_reason_bad_offset = stats_raw_data['reason-bad-offset'],
                              drop_reason_bad_timestamp = stats_raw_data['reason-bad-timestamp'],
                              drop_reason_congestion = stats_raw_data['reason-congestion'],
                              drop_reason_connection_limit = stats_raw_data['reason-connection-limit'],
                              drop_reason_drop_by_loadbalancer = stats_raw_data['reason-drop-by-loadbalancer'],
                              drop_reason_failed_to_copy_pkt = stats_raw_data['reason-failed-to-copy-pkt'],
                              drop_reason_fragment = stats_raw_data['reason-fragment'],
                              drop_reason_half_open_tcp_max = stats_raw_data['reason-half-open-tcp-max'],
                              drop_reason_icmp_max = stats_raw_data['reason-icmp-max'],
                              drop_reason_inactive = stats_raw_data['reason-inactive'],
                              drop_reason_ip_option = stats_raw_data['reason-ip-option'],
                              drop_reason_memory = stats_raw_data['reason-memory'],
                              drop_reason_nat_conn_limit = stats_raw_data['reason-nat-conn-limit'],
                              drop_reason_nat64_no_frgm_support = stats_raw_data['reason-nat64-no-frgm-support'],
                              drop_reason_normalize = stats_raw_data['reason-normalize'],
                              drop_reason_other_max = stats_raw_data['reason-other-max'],
                              drop_reason_proto_cksum = stats_raw_data['reason-proto-cksum'],
                              drop_reason_queued_frag = stats_raw_data['reason-queued-frag'],
                              drop_reason_redirect_iface_null = stats_raw_data['reason-redirect-iface-null'],
                              drop_reason_rst_sent = stats_raw_data['reason-rst-sent'],
                              drop_reason_short = stats_raw_data['reason-short'],
                              drop_reason_spoofguard = stats_raw_data['reason-spoofguard'],
                              drop_reason_src_limit = stats_raw_data['reason-src-limit'],
                              drop_reason_state_insert = stats_raw_data['reason-state-insert'],
                              drop_reason_state_limit = stats_raw_data['reason-state-limit'],
                              drop_reason_state_mismatch = stats_raw_data['reason-state-mismatch'],
                              drop_reason_state_reuse = stats_raw_data['reason-state-reuse'],
                              drop_reason_synproxy = stats_raw_data['reason-synproxy'],
                              drop_reason_tcp_flags = stats_raw_data['reason-tcp-flags'],
                              drop_reason_tcp_seqnum = stats_raw_data['reason-tcp-seqnum'],
                              drop_reason_translation = stats_raw_data['reason-translation'],
                              drop_reason_tun_fail = stats_raw_data['reason-tun-fail'],
                              drop_reason_udp_max = stats_raw_data['reason-udp-max'],
                              drop_reason_update_state = stats_raw_data['reason-update-state'],
                              rx_ipv4_packets = stats_raw_data['packets-ipv4-in-pass'],
                              rx_ipv4_bytes = stats_raw_data['bytes-ipv4-in-pass'],
                              rx_ipv4_drop_packets = stats_raw_data['packets-ipv4-in-drop'],
                              rx_ipv4_drop_bytes = stats_raw_data['bytes-ipv4-in-drop'],
                              tx_ipv4_packets = stats_raw_data['packets-ipv4-out-pass'],
                              tx_ipv4_bytes = stats_raw_data['bytes-ipv4-out-pass'],
                              tx_ipv4_drop_packets = stats_raw_data['packets-ipv4-out-drop'],
                              tx_ipv4_drop_bytes = stats_raw_data['bytes-ipv4-out-drop'],
                              rx_ipv6_packets = stats_raw_data['packets-ipv6-in-pass'],
                              rx_ipv6_bytes = stats_raw_data['bytes-ipv6-in-pass'],
                              rx_ipv6_drop_packets = stats_raw_data['packets-ipv6-in-drop'],
                              rx_ipv6_drop_bytes = stats_raw_data['bytes-ipv6-in-drop'],
                              tx_ipv6_packets = stats_raw_data['packets-ipv6-out-pass'],
                              tx_ipv6_bytes = stats_raw_data['bytes-ipv6-out-pass'],
                              tx_ipv6_drop_packets = stats_raw_data['packets-ipv6-out-drop'],
                              tx_ipv6_drop_bytes = stats_raw_data['bytes-ipv6-out-drop'])
                except KeyError:
                    logger.info("Keyerror, not sending edge FW stats to SHA")
                    continue
                sub_msg = msg.edge_fw.add()
                sub_msg.CopyFrom(tmp_msg)
                Exportdata(data = msg, exports = ['tsdb_metrics'])

profile = data_store['profile']
edge_fw_stats = data_store['metrics']['nsx.edge.edge-fw-stats']
report_edge_fw_stats(edge_fw_stats, profile)
