# Copyright (C) 2021 VMware, Inc. All rights reserved.
# -- VMware Confidential

lb_status_dict = {"ready" : 0, "not_ready": 1}
dlb_status_dict = {"ready" : 0, "partially ready": 1, "not ready": 2}
lb_pool_status_dict = {"up" : 0, "partially_up" : 1, "down" : 2, "primary_down" : 3, "unknown" : 4}
lb_vs_status_dict = {"up" : 0, "partially_up" : 1, "down" : 2, "primary_down" : 3, "disabled" : 4}
lb_threshold = {"cpu" : {"sensitivity" : 80, "threshold" : 95}, "capacity" : {"sensitivity" : 80, "threshold" : 95}}
lb_threshold_dict = {"normal": 0, "sensitivity" : 1, "threshold": 2}

def lb_threshold_alarm(lb_status_report, lb_status_key, threshold_key, metrics_key):
    if lb_status_report[lb_status_key] > lb_threshold[threshold_key]["threshold"]:
        lb_status_report[metrics_key] = lb_threshold_dict["threshold"]
    elif lb_status_report[lb_status_key] > lb_threshold[threshold_key]["sensitivity"]:
        lb_status_report[metrics_key] = lb_threshold_dict["sensitivity"]
    else:
        lb_status_report[metrics_key] = lb_threshold_dict["normal"]

def report_lb_status(lb_status_list, lb_capacity_factor, source):
    try:
        lb_status_report = {}
        lb_capacity = 0.0
        for lb_status in lb_status_list['lbs']:
            metrix_key_prefix = 'edge.{}.{}.lb.{}'.format(lb_status['sr_info']['sr_type'], lb_status['sr_info']['sr_name'], lb_status['display_name'])
            lb_capacity += 1 / float(lb_capacity_factor[lb_status['size']])
            for lb_pool in lb_status['pools']:
                lb_status_report['{}.pool.{}.status'.format(metrix_key_prefix, lb_pool['display_name'])] = lb_pool_status_dict[lb_pool['status']]
            for lb_vs in lb_status['virtual_servers']:
                lb_status_report['{}.vs.{}.status'.format(metrix_key_prefix, lb_vs['display_name'])] = lb_vs_status_dict[lb_vs['status']]
            lb_status_report['{}.status'.format(metrix_key_prefix)] = lb_status_dict[lb_status['status']]
            lb_status_report['{}.cpu_usage'.format(metrix_key_prefix)] = int(lb_status['cpu_usage'])
            lb_threshold_alarm(lb_status_report = lb_status_report, lb_status_key = '{}.cpu_usage'.format(metrix_key_prefix),
                               threshold_key = 'cpu', metrics_key = '{}.cpu_alarm'.format(metrix_key_prefix))
        lb_status_report['edge.LB_capacity'] = int(100 * lb_capacity)
        lb_threshold_alarm(lb_status_report = lb_status_report, lb_status_key = 'edge.LB_capacity',
                           threshold_key = 'capacity', metrics_key = 'edge.LB_capacity.alarm')
        Exportdata(data = lb_status_report, exports = ['wavefront'], source = source)
    except Exception as e:
        logger.warn("Failed to get load balancer status {}".format(e))

def report_dlb_status(lb_status_list):
    try:
        lb_status_report = {}
        for lb_status in lb_status_list['lbs']:
            lb_status_report['esx.LB_{}.status'.format(lb_status['uuid'])] = dlb_status_dict[lb_status['status']]
        Exportdata(data = lb_status_report, exports = ['wavefront'])
    except Exception as e:
        logger.warn("Failed to get load balancer status {}".format(e))

lb_status = data_store['metrics']['nsx.lb-status']
if lb_status is not None and lb_status['value'] is not None:
    if lb_status['value']['lb_type'] == 'lb':
        report_lb_status(lb_status['value']['context'], lb_status['value']['lb_capacity'], lb_status['value']['source'])
    elif lb_status['value']['lb_type'] == 'dlb':
        report_dlb_status(lb_status['value']['context'])
    else:
        logger.warn("Invalid lb_type {}".format(lb_status['value']['lb_type']))
else:
    logger.warn("No changes needs be reported.")
